@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+// Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -72,6 +72,16 @@ var packUint32big = function packUint32big (value) {
                        (value >>> 8) & 0xff,
                        value & 0xff]);
 };
+var nodejs_version = process.versions['node'].split('.').map(s => parseInt(s));
+var nodejsVersionAfter = function nodejsVersionAfter (s) {
+    var v = s.split('.').map(s => parseInt(s));
+    for (var i = 0; i < v.length; i++) {
+        if (nodejs_version[i] > v[i]) {
+            return true;
+        }
+    }
+    return false;
+};
 
 var InvalidInputException = function InvalidInputException () {
     var error = new Error('Invalid Input');
@@ -203,7 +213,7 @@ CloudI.API = function API (thread_index, callback) {
     else {
         throw new InvalidInputException();
     }
-    if (process.versions['node'] > '0.12.1') {
+    if (nodejsVersionAfter('0.12.1')) {
         API._s_in = new net.Socket({fd: (thread_index + 3),
                                     readable: true,
                                     writable: true});
@@ -217,6 +227,7 @@ CloudI.API = function API (thread_index, callback) {
     }
     API._initialization_complete = false;
     API._terminate = false;
+    API._terminate_callback = undefined;
     API._size = parseInt(buffer_size_str);
     API._callbacks = {};
     API._timeout_terminate = 1000; // TIMEOUT_TERMINATE_MIN
@@ -257,22 +268,22 @@ CloudI.API = function API (thread_index, callback) {
             if (err instanceof InvalidInputException ||
                 err instanceof MessageDecodingException) {
                 CloudI.stderr_write(err.stack + '\n');
-                API._terminate = true;
+                API._poll_terminate();
                 process.exit(1);
             }
             else if (err instanceof TerminateException) {
-                API._terminate = true;
+                API._poll_terminate();
             }
             else {
                 API._exception(err);
-                API._terminate = true;
+                API._poll_terminate();
                 process.exit(1);
             }
         }
     });
     API._s_in.on('error', function(err) {
         API._exception(err);
-        API._terminate = true;
+        API._poll_terminate();
         process.exit(1);
     });
     API._send(new Erlang.OtpErlangAtom('init'));
@@ -559,18 +570,18 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 if (err instanceof InvalidInputException ||
                     err instanceof MessageDecodingException) {
                     CloudI.stderr_write(err.stack + '\n');
-                    API._terminate = true;
+                    API._poll_terminate();
                     process.exit(1);
                 }
                 else if (err instanceof TerminateException) {
-                    API._terminate = true;
+                    API._poll_terminate();
                 }
                 else if (err instanceof ReturnAsyncException) {
                     return;
                 }
                 else if (err instanceof ReturnSyncException) {
                     CloudI.stderr_write(err.stack + '\n');
-                    API._terminate = true;
+                    API._poll_terminate();
                     process.exit(1);
                 }
                 else if (err instanceof ForwardAsyncException) {
@@ -578,7 +589,7 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 }
                 else if (err instanceof ForwardSyncException) {
                     CloudI.stderr_write(err.stack + '\n');
-                    API._terminate = true;
+                    API._poll_terminate();
                     process.exit(1);
                 }
                 else {
@@ -638,15 +649,15 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 if (err instanceof InvalidInputException ||
                     err instanceof MessageDecodingException) {
                     CloudI.stderr_write(err.stack + '\n');
-                    API._terminate = true;
+                    API._poll_terminate();
                     process.exit(1);
                 }
                 else if (err instanceof TerminateException) {
-                    API._terminate = true;
+                    API._poll_terminate();
                 }
                 else if (err instanceof ReturnAsyncException) {
                     CloudI.stderr_write(err.stack + '\n');
-                    API._terminate = true;
+                    API._poll_terminate();
                     process.exit(1);
                 }
                 else if (err instanceof ReturnSyncException) {
@@ -654,7 +665,7 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 }
                 else if (err instanceof ForwardAsyncException) {
                     CloudI.stderr_write(err.stack + '\n');
-                    API._terminate = true;
+                    API._poll_terminate();
                     process.exit(1);
                 }
                 else if (err instanceof ForwardSyncException) {
@@ -835,7 +846,7 @@ CloudI.API.prototype._poll_request = function (data) {
                 Erlang.binary_to_term(pid, function (err, pid_object) {
                     if (err) {
                         API._exception(err);
-                        API._terminate = true;
+                        API._poll_terminate();
                         process.exit(1);
                     }
                     else {
@@ -975,23 +986,51 @@ CloudI.API.prototype._poll_wait = function (f) {
     }
 };
 
+CloudI.API.prototype._poll_terminate = function () {
+    var API = this;
+    API._terminate = true;
+    API._s_in.destroy();
+    if (API._terminate_callback !== undefined) {
+        API._terminate_callback(false);
+    }
+};
+
+CloudI.API.prototype._poll_blocked = function () {
+    var API = this;
+    if (API._terminate) {
+        return;
+    }
+    timers.setTimeout(function () {
+        API._poll_blocked();
+    }, 500);
+};
+
 CloudI.API.prototype._poll_block = function (callback, timeout) {
     var API = this;
     if (API._terminate) {
-        callback(false);
+        return;
     }
     else if (timeout == 0) {
+        API._terminate_callback = undefined;
         callback(true);
     }
     else if (timeout > 0) {
-        timers.setTimeout(function () {
-            API._poll_block(callback, 0);
-        }, timeout);
+        var timeout_interval = 500;
+        API._terminate_callback = callback;
+        if (timeout > timeout_interval) {
+            timers.setTimeout(function () {
+                API._poll_block(callback, timeout - timeout_interval);
+            }, timeout_interval);
+        }
+        else {
+            timers.setTimeout(function () {
+                API._poll_block(callback, 0);
+            }, timeout);
+        }
     }
     else {
-        timers.setTimeout(function () {
-            API._poll_block(callback, -1);
-        }, 0);
+        API._terminate_callback = callback;
+        API._poll_blocked();
     }
 };
 
@@ -1035,7 +1074,8 @@ CloudI.API.prototype._text_key_value_parse = function (text) {
 };
 
 CloudI.API.prototype.info_key_value_parse = function (message_info) {
-    return CloudI.API.prototype._text_key_value_parse(message_info);
+    var API = this;
+    return API._text_key_value_parse(message_info);
 };
 
 CloudI.API.prototype._exception = function (err) {
@@ -1050,7 +1090,7 @@ CloudI.API.prototype._send = function (terms) {
     Erlang.term_to_binary(terms, function (err, data) {
         if (err) {
             API._exception(err);
-            API._terminate = true;
+            API._poll_terminate();
             process.exit(1);
             return;
         }
