@@ -322,7 +322,7 @@ all_ext(<<"gqf">>) -> {<<"application">>, <<"vnd.grafeq">>, []};
 all_ext(<<"gqs">>) -> {<<"application">>, <<"vnd.grafeq">>, []};
 all_ext(<<"gram">>) -> {<<"application">>, <<"srgs">>, []};
 all_ext(<<"gramps">>) -> {<<"application">>, <<"x-gramps-xml">>, []};
-all_ext(<<"cloudi_x_gre">>) -> {<<"application">>, <<"vnd.geometry-explorer">>, []};
+all_ext(<<"gre">>) -> {<<"application">>, <<"vnd.geometry-explorer">>, []};
 all_ext(<<"grv">>) -> {<<"application">>, <<"vnd.groove-injector">>, []};
 all_ext(<<"grxml">>) -> {<<"application">>, <<"srgs+xml">>, []};
 all_ext(<<"gsf">>) -> {<<"application">>, <<"x-font-ghostscript">>, []};
@@ -427,7 +427,7 @@ all_ext(<<"list3820">>) -> {<<"application">>, <<"vnd.ibm.modcap">>, []};
 all_ext(<<"listafp">>) -> {<<"application">>, <<"vnd.ibm.modcap">>, []};
 all_ext(<<"list">>) -> {<<"text">>, <<"plain">>, []};
 all_ext(<<"lnk">>) -> {<<"application">>, <<"x-ms-shortcut">>, []};
-all_ext(<<"cloudi_x_log">>) -> {<<"text">>, <<"plain">>, []};
+all_ext(<<"log">>) -> {<<"text">>, <<"plain">>, []};
 all_ext(<<"lostxml">>) -> {<<"application">>, <<"lost+xml">>, []};
 all_ext(<<"lrf">>) -> {<<"application">>, <<"octet-stream">>, []};
 all_ext(<<"lrm">>) -> {<<"application">>, <<"vnd.ms-lrm">>, []};
