@@ -9,7 +9,7 @@
 %% -------------------------------------------------------------------
 
 
-%% @doc API and behaviour for metrics instances
+%% @doc API and behaviour for cloudi_x_metrics instances
 %%
 %% <h2>Predefined templates</h2>
 %%
@@ -73,10 +73,13 @@
 -compile(inline).
 
 -include("cloudi_x_exometer.hrl").
--include("cloudi_x_log.hrl").
 
 -type name()        :: list().
--type type()        :: atom().
+-type type()        :: atom()
+                     | {function, M :: atom(), F :: atom()}
+                     | {function, M :: atom(), F :: atom(), ArgSpec :: list(),
+                        Type :: atom(), DataPoints :: list()}
+                     | {Type :: atom(), Arg :: any()}.
 -type status()      :: enabled | disabled.
 -type options()     :: [{atom(), any()}].
 -type value()       :: any().
@@ -97,8 +100,7 @@ start() ->
 
 %% @doc Stop cloudi_x_exometer and dependent apps (for testing).
 stop() ->
-    application:stop(cloudi_x_exometer_core),
-    application:stop(cloudi_x_lager).
+    application:stop(cloudi_x_exometer_core).
 
 -spec new(name(), type()) -> ok.
 %% @equiv new(Name, Type, [])
@@ -106,7 +108,7 @@ new(Name, Type) ->
     new(Name, Type, []).
 
 -spec new(name(), type(), options()) -> ok.
-%% @doc Create a new metrics entry.
+%% @doc Create a new cloudi_x_metrics entry.
 %%
 %% `Name' must be a list of terms (e.g. atoms). `Type' must be either one
 %% of the built-in types, or match a predefined template.
@@ -163,7 +165,7 @@ propose(Name, Type, Opts) when is_list(Name), is_list(Opts) ->
     cloudi_x_exometer_admin:propose(Name, Type, Opts).
 
 -spec re_register(name(), type(), options()) -> ok.
-%% @doc Create a new metrics entry, overwrite any old entry.
+%% @doc Create a new cloudi_x_metrics entry, overwrite any old entry.
 %%
 %% This function behaves as {@link new/3}, but will not fail if an entry
 %% with the same name already exists. Instead, the old entry will be replaced
@@ -240,7 +242,10 @@ update_(Name, Value) when is_list(Name) ->
 
 update_ok(St, #cloudi_x_exometer_entry{name = Name}, Value)
   when St band 2#10 =:= 2#10 ->
-    try exometer_event ! {updated, Name, Value}
+    try begin
+	    exometer_event ! {updated, Name, Value},
+	    ok
+	end
     catch
         _:_ -> ok
     end;
@@ -289,7 +294,7 @@ fast_incr(0, _, _) ->
 %% @doc Fetch the current value of the metric.
 %%
 %% For a built-in counter, the value returned is the sum of all counter
-%% instances (one per scheduler). For plugin metrics, the callback module is
+%% instances (one per scheduler). For plugin cloudi_x_metrics, the callback module is
 %% responsible for providing the value. If the metric has a specified
 %% (non-zero) cache lifetime, and a value resides in the cache, the cached
 %% value will be returned.
@@ -385,13 +390,18 @@ get_cached_value_(#cloudi_x_exometer_entry{name = Name,
                     end, {[],[]}, DataPoints),
 
     %% Go through all cache misses and retreive their actual values.
-    Result = get_value_(E#cloudi_x_exometer_entry { cache = 0 }, Uncached),
-
-    %% Update the cache with all the shiny new values retrieved.
-    [ cloudi_x_exometer_cache:write(Name, DataPoint1, Value1, CacheTTL)
-      || { DataPoint1, Value1 } <- Result],
-    All = Result ++ Cached,
-    [{_,_} = lists:keyfind(DP, 1, All) || DP <- DataPoints].
+    case get_value_(E#cloudi_x_exometer_entry { cache = 0 }, Uncached) of
+        {error, unavailable} = Result -> 
+            %% a function entry returns this in exception case.
+            %% see `cloudi_x_exometer_function:get_value/4`
+            Result;
+        Result ->
+            %% Update the cache with all the shiny new values retrieved.
+            [ cloudi_x_exometer_cache:write(Name, DataPoint1, Value1, CacheTTL)
+              || { DataPoint1, Value1 } <- Result],
+            All = Result ++ Cached,
+            [{_,_} = lists:keyfind(DP, 1, All) || DP <- DataPoints]
+    end.
 
 
 
@@ -405,7 +415,7 @@ delete(Name) when is_list(Name) ->
 %%
 %% Probes often take care of data sampling using a configured sample
 %% interval. This function provides a way to explicitly tell a probe to
-%% take a sample. The operation is asynchronous. For other metrics, the
+%% take a sample. The operation is asynchronous. For other cloudi_x_metrics, the
 %% operation likely has no effect, and will return `ok'.
 %% @end
 sample(Name)  when is_list(Name) ->
@@ -650,33 +660,34 @@ update_entry_elems(Name, Elems) ->
 %% * `cache' - The cache lifetime
 %% * `status' - Operational status: `enabled' or `disabled'
 %% * `timestamp' - When the metric was last reset/initiated
-%% * `datapoitns' - Data points available for retrieval with get_value()
+%% * `datapoints' - Data points available for retrieval with get_value()
 %% * `options' - Options passed to the metric at creation (or via setopts())
 %% * `ref' - Instance-specific reference; usually a pid (probe) or undefined
 %% @end
-info(#cloudi_x_exometer_entry{} = E, Item) ->
-    info_(E, Item);
+info(#cloudi_x_exometer_entry{status = Status} = E, Item) ->
+    info_(E, Item, info_status(Status));
 info(Name, Item) ->
     case ets:lookup(cloudi_x_exometer_util:table(), Name) of
-        [#cloudi_x_exometer_entry{} = E] ->
-            info_(E, Item);
+        [#cloudi_x_exometer_entry{status = Status0} = E] ->
+            Status = info_status(Status0),
+            info_(E, Item, Status);
         _ ->
             undefined
     end.
 
-info_(E, Item) ->
-    case Item of
-        name      -> E#cloudi_x_exometer_entry.name;
-        type      -> E#cloudi_x_exometer_entry.type;
-        module    -> E#cloudi_x_exometer_entry.module;
-        value     -> get_value_(E,[]);
-        cache     -> E#cloudi_x_exometer_entry.cache;
-        status    -> info_status(E#cloudi_x_exometer_entry.status);
-        timestamp -> E#cloudi_x_exometer_entry.timestamp;
-        options   -> E#cloudi_x_exometer_entry.options;
-        ref       -> E#cloudi_x_exometer_entry.ref;
-        entry     -> E;
-        datapoints-> cloudi_x_exometer_util:get_datapoints(E);
+info_(E, Item, Status) ->
+    case {Item, Status} of
+        {name,             _} -> E#cloudi_x_exometer_entry.name;
+        {type,             _} -> E#cloudi_x_exometer_entry.type;
+        {module,           _} -> E#cloudi_x_exometer_entry.module;
+        {value,      enabled} -> get_value_(E, cloudi_x_exometer_util:get_datapoints(E));
+        {cache,      enabled} -> E#cloudi_x_exometer_entry.cache;
+        {status,           _} -> Status;
+        {timestamp,  enabled} -> E#cloudi_x_exometer_entry.timestamp;
+        {options,          _} -> E#cloudi_x_exometer_entry.options;
+        {ref,              _} -> E#cloudi_x_exometer_entry.ref;
+        {entry,            _} -> E;
+        {datapoints, enabled} -> cloudi_x_exometer_util:get_datapoints(E);
         _ -> undefined
     end.
 
@@ -698,33 +709,26 @@ datapoints(D, _) when is_list(D) ->
 %% @doc Returns a list of info items for Metric, see {@link info/2}.
 info(Name) ->
     case ets:lookup(cloudi_x_exometer_util:table(), Name) of
-        [#cloudi_x_exometer_entry{} = E0] ->
-            E = info_set_status(E0),
+        [#cloudi_x_exometer_entry{status = Status0} = E] ->
+            Status = info_status(Status0),
             Flds = record_info(fields, cloudi_x_exometer_entry),
-            lists:keyreplace(value, 1,
-                             lists:zip(Flds, tl(tuple_to_list(E))) ++
-                                 [ {datapoints,
-                                    cloudi_x_exometer_util:get_datapoints(E)}],
-                             {value, get_value_(E, [])});
+            lists:map(fun(Item) -> {Item, info_(E, Item, Status)} end, Flds);
         _ ->
             undefined
     end.
 
-info_set_status(#cloudi_x_exometer_entry{status = S} = E) ->
-    E#cloudi_x_exometer_entry{status = info_status(S)}.
-
 -spec find_entries([any() | '_']) -> [{name(), type(), status()}].
-%% @doc Find metrics based on a name prefix pattern.
+%% @doc Find cloudi_x_metrics based on a name prefix pattern.
 %%
-%% This function will find and return metrics whose name matches the given
-%% prefix. For example `[kvdb, kvdb_conf, Table]' would match any metrics
+%% This function will find and return cloudi_x_metrics whose name matches the given
+%% prefix. For example `[kvdb, kvdb_conf, Table]' would match any cloudi_x_metrics
 %% tied to the given table in the `kvdb_conf' database.
 %%
 %% It is possible to insert wildcards:
 %% <code>[kvdb, kvdb_conf, '_', write]</code> would match
-%% `write'-related metrics in all tables of the `kvdb_conf' database.
+%% `write'-related cloudi_x_metrics in all tables of the `kvdb_conf' database.
 %%
-%% The format of the returned metrics is `[{Name, Type, Status}]'.
+%% The format of the returned cloudi_x_metrics is `[{Name, Type, Status}]'.
 %% @end
 find_entries(Path) ->
     Pat = Path ++ '_',
@@ -748,11 +752,11 @@ get_values(Path) ->
       end, [], Entries).
 
 -spec select(ets:match_spec()) -> list().
-%% @doc Perform an `ets:select()' on the set of metrics.
+%% @doc Perform an `ets:select()' on the set of cloudi_x_metrics.
 %%
-%% This function operates on a virtual structure representing the metrics,
+%% This function operates on a virtual structure representing the cloudi_x_metrics,
 %% but otherwise works as a normal `select()'. The representation of the
-%% metrics is `{Name, Type, Status}'.
+%% cloudi_x_metrics is `{Name, Type, Status}'.
 %% @end
 select(Pattern) ->
     ets:select(?EXOMETER_ENTRIES, [pattern(P) || P <- Pattern]).
@@ -764,7 +768,7 @@ select_count(Pattern) ->
     ets:select_count(?EXOMETER_ENTRIES, [pattern(P) || P <- Pattern]).
 
 -spec select(ets:match_spec(), pos_integer() | infinity) -> {list(), _Cont}.
-%% @doc Perform an `ets:select()' with a Limit on the set of metrics.
+%% @doc Perform an `ets:select()' with a Limit on the set of cloudi_x_metrics.
 %%
 %% This function is equivalent to {@link select/1}, but also takes a limit.
 %% After `Limit' number of matches, the function returns the matches and a
