@@ -40,7 +40,7 @@
 %% Supported options:
 %%
 %% * `time_span' (default: `60000') size of the window in milliseconds.
-%% * `slot_period' (default: `1000') size of the time slots in milliseconds.
+%% * `slot_period' (default: `10') size of the time slots in milliseconds.
 %% * `histogram_module' (default: `cloudi_x_exometer_slot_slide').
 %% * `truncate' (default: `true') whether to truncate the datapoint values.
 %%     Supported values: `true | false | round', where `round' means to round
@@ -91,7 +91,7 @@
 
 -record(st, {name,
              slide = undefined, %%
-             slot_period = 1000, %% msec
+             slot_period = 10, %% msec
              time_span = 60000, %% msec
              truncate = true,
              histogram_module = cloudi_x_exometer_slot_slide,
@@ -112,10 +112,7 @@ probe_init(Name, _Type, Options) ->
     {ok, init_state(Name, Options)}.
 
 init_state(Name, Options) ->
-    St = process_opts(#st{name = Name},
-                      [{histogram_module, cloudi_x_exometer_slot_slide},
-                       {time_span, 60000},
-                       {slot_period, 10}] ++ Options),
+    St = process_opts(#st{name = Name}, Options),
     Slide = (St#st.histogram_module):new(St#st.time_span,
                                          St#st.slot_period,
                                          fun average_sample/3,
