@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2009-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2009-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
+%%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2009-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_logger).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(gen_server).
 
@@ -50,7 +50,10 @@
          redirect_update/1,
          fatal/8, error/8, warn/8, info/8, debug/8, trace/8,
          metadata_get/0, metadata_set/1,
-         format/2, format/3]).
+         format/2, format/3,
+         microseconds_to_string/1,
+         milliseconds_to_string/1,
+         seconds_to_string/1]).
 
 %% gen_server callbacks
 -export([init/1,
@@ -465,6 +468,70 @@ format(Msg, _Config, _) ->
                 Module, Line, Function, undefined,
                 ExtraMetaData, LogMessage).
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Create an ISO8601 timestamp from microseconds since the UNIX epoch.===
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec microseconds_to_string(TotalMicroSeconds :: non_neg_integer()) ->
+    string().
+
+microseconds_to_string(TotalMicroSeconds) ->
+    TotalSeconds = TotalMicroSeconds div 1000000,
+    MegaSeconds = TotalSeconds div 1000000,
+    Seconds = TotalSeconds - MegaSeconds * 1000000,
+    MicroSeconds = TotalMicroSeconds - TotalSeconds * 1000000,
+    timestamp_iso8601({MegaSeconds, Seconds, MicroSeconds}).
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Create an ISO8601 timestamp from milliseconds since the UNIX epoch.===
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec milliseconds_to_string(TotalMilliSeconds :: non_neg_integer()) ->
+    string().
+
+milliseconds_to_string(TotalMilliSeconds) ->
+    TotalSeconds = TotalMilliSeconds div 1000,
+    MegaSeconds = TotalSeconds div 1000000,
+    Seconds = TotalSeconds - MegaSeconds * 1000000,
+    MilliSeconds = TotalMilliSeconds - TotalSeconds * 1000,
+    [DateYYYY0, DateYYYY1, DateYYYY2, DateYYYY3, $-,
+     DateMM0, DateMM1, $-, DateDD0, DateDD1, $T,
+     TimeHH0, TimeHH1, $:, TimeMM0, TimeMM1, $:, TimeSS0, TimeSS1, $.,
+     MicroSeconds0, MicroSeconds1, MicroSeconds2, _, _, _,
+     $Z] = timestamp_iso8601({MegaSeconds, Seconds, MilliSeconds * 1000}),
+    [DateYYYY0, DateYYYY1, DateYYYY2, DateYYYY3, $-,
+     DateMM0, DateMM1, $-, DateDD0, DateDD1, $T,
+     TimeHH0, TimeHH1, $:, TimeMM0, TimeMM1, $:, TimeSS0, TimeSS1, $.,
+     MicroSeconds0, MicroSeconds1, MicroSeconds2, $Z].
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Create an ISO8601 timestamp from seconds since the UNIX epoch.===
+%% (The UNIX epoch is 1970-01-01T00:00:00Z)
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec seconds_to_string(TotalSeconds :: non_neg_integer()) ->
+    string().
+
+seconds_to_string(TotalSeconds) ->
+    MegaSeconds = TotalSeconds div 1000000,
+    Seconds = TotalSeconds - MegaSeconds * 1000000,
+    [DateYYYY0, DateYYYY1, DateYYYY2, DateYYYY3, $-,
+     DateMM0, DateMM1, $-, DateDD0, DateDD1, $T,
+     TimeHH0, TimeHH1, $:, TimeMM0, TimeMM1, $:, TimeSS0, TimeSS1, $.,
+     _, _, _, _, _, _,
+     $Z] = timestamp_iso8601({MegaSeconds, Seconds, 0}),
+    [DateYYYY0, DateYYYY1, DateYYYY2, DateYYYY3, $-,
+     DateMM0, DateMM1, $-, DateDD0, DateDD1, $T,
+     TimeHH0, TimeHH1, $:, TimeMM0, TimeMM1, $:, TimeSS0, TimeSS1, $Z].
+
 %%%------------------------------------------------------------------------
 %%% Callback functions from gen_server
 %%%------------------------------------------------------------------------
@@ -563,7 +630,7 @@ handle_call({Level, Timestamp, Node, Pid,
             {stop, Reason, ok, StateNext}
     end;
 handle_call(Request, _, State) ->
-    {stop, cloudi_string:format("Unknown call \"~p\"~n", [Request]),
+    {stop, cloudi_string:format("Unknown call \"~w\"", [Request]),
      error, State}.
 
 handle_cast({set, LoggingConfig}, State) ->
@@ -654,7 +721,7 @@ handle_cast({Level, Timestamp, Node, Pid,
             {stop, Reason, StateNext}
     end;
 handle_cast(Request, State) ->
-    {stop, cloudi_string:format("Unknown cast \"~p\"~n", [Request]), State}.
+    {stop, cloudi_string:format("Unknown cast \"~w\"", [Request]), State}.
 
 handle_info({'DOWN', _, process, Process, Info},
             #state{syslog = Syslog} = State) ->
@@ -704,7 +771,7 @@ handle_info({'CHANGE', Monitor, time_offset, clock_service, TimeOffset},
             {stop, Reason, StateNew}
     end;
 handle_info(Request, State) ->
-    {stop, cloudi_string:format("Unknown info \"~p\"~n", [Request]), State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
 terminate(_, #state{fd = Fd,
                     stdout = StdoutPort,
@@ -999,11 +1066,11 @@ log_message_formatter_call(Level, Timestamp, Node, Pid,
                     MetaData, LogMessage),
     try Formatter:format(Msg, FormatterConfig)
     catch
-        ErrorType:Error ->
+        ?STACKTRACE(ErrorType, Error, ErrorStackTrace)
             ErrorMessage = cloudi_string:
                            format_to_binary("formatter(~p) ~p ~p~n~p",
                                             [Formatter, ErrorType, Error,
-                                             erlang:get_stacktrace()]),
+                                             ErrorStackTrace]),
             [format_line(Level, Timestamp, Node, Pid,
                          Module, Line, Function, Arity,
                          MetaData, LogMessage),
@@ -1035,11 +1102,11 @@ log_message_formatter_call(Level, Timestamp, Node, Pid,
             format_line(Level, Timestamp, Node, Pid,
                         Module, Line, Function, Arity,
                         MetaData, LogMessage);
-        ErrorType:Error ->
+        ?STACKTRACE(ErrorType, Error, ErrorStackTrace)
             ErrorMessage = cloudi_string:
                            format_to_binary("output(~p) ~p ~p~n~p",
                                             [Output, ErrorType, Error,
-                                             erlang:get_stacktrace()]),
+                                             ErrorStackTrace]),
             [format_line(Level, Timestamp, Node, Pid,
                          Module, Line, Function, Arity,
                          MetaData, LogMessage),
@@ -1490,7 +1557,7 @@ timestamp_increment({MegaSecs, Secs, MicroSecs}) ->
 -define(INTERFACE_MODULE_HEADER,
     "
     -module(cloudi_core_i_logger_interface).
-    -author('mjtruog [at] gmail (dot) com').
+    -author('mjtruog at protonmail dot com').
     -export([fatal/6, error/6, warn/6, info/6, debug/6, trace/6,
              fatal_sync/6, error_sync/6, warn_sync/6,
              info_sync/6, debug_sync/6, trace_sync/6,
