@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,13 +29,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_os_spawn).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behavior(gen_server).
 
@@ -99,8 +99,8 @@ handle_call({call, Command, Msg}, Client,
     end;
 
 handle_call(Request, _, State) ->
-    ?LOG_ERROR("Unknown call \"~p\"~n", [Request]),
-    {stop, "Unknown call", State}.
+    {stop, cloudi_string:format("Unknown call \"~w\"", [Request]),
+     error, State}.
 
 %% handle asynchronous function calls on the port driver
 handle_cast({call, _, Msg},
@@ -116,8 +116,7 @@ handle_cast({call, _, Msg},
     {noreply, State};
 
 handle_cast(Request, State) ->
-    ?LOG_WARN("Unknown cast \"~p\"~n", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown cast \"~w\"", [Request]), State}.
 
 %% port exited with a fatal error/signal
 handle_info({Port, {exit_status, Status}},
@@ -256,8 +255,7 @@ handle_info({ReplyRef, _}, State) when is_reference(ReplyRef) ->
     {noreply, State};
 
 handle_info(Request, State) ->
-    ?LOG_WARN("Unknown info \"~p\"~n", [Request]),
-    {noreply, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
 terminate(_, #state{port = Port}) when is_port(Port) ->
     catch erlang:port_close(Port),
