@@ -22,11 +22,19 @@
 %%%
 %%% Only one instance of cloudi_queue is necessary within a service's state
 %%% due to the transaction id being globally unique.
+%%%
+%%% If retries are enabled (by default, no retries are attempted), it is
+%%% possible that a service request will be delivered more than once
+%%% (e.g., if the service request is processed but the response is not
+%%%  sent before a timeout occurs).  Each retry sends a service request
+%%% with the same transaction id, so it may be used to detect a duplicate
+%%% receive (or a separate unique id could be used from within the
+%%% RequestInfo or Request data) by checking and modifying service state.
 %%% @end
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -46,13 +54,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
+%%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2015-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_queue).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([byte_size/2,
@@ -83,6 +91,7 @@
          size/2,
          timeout/3]).
 
+-include("cloudi_service.hrl").
 -include("cloudi_core_i_constants.hrl").
 
 -record(request,
@@ -128,6 +137,8 @@
         retry :: non_neg_integer(),
         retry_delay :: non_neg_integer(),
         ordered :: boolean(),
+        timeout_default :: cloudi_service:timeout_milliseconds(),
+        priority_default :: cloudi_service:priority(),
         word_size :: pos_integer(),
         service = undefined :: undefined | pid(),
         validate_request_info :: undefined | fun((any()) -> boolean()),
@@ -143,13 +154,19 @@
         requests = #{} :: requests()
     }).
 
--include("cloudi_service.hrl").
-
 -define(DEFAULT_RETRY,                                0). % see below:
         % a retry doesn't count as a failure, until it fails completely
         % (i.e., hit the max retry count or send returns an error)
 -define(DEFAULT_RETRY_DELAY,                          0). % milliseconds
 -define(DEFAULT_ORDERED,                          false).
+-define(DEFAULT_TIMEOUT_DEFAULT,              undefined).
+        % provide a default timeout that will be used instead of
+        % the timeout_async service configuration value when the
+        % timeout is provided as undefined
+-define(DEFAULT_PRIORITY_DEFAULT,             undefined).
+        % provide a default priority that will be used instead of
+        % the service configuration option priority_default when the
+        % priority is provided as undefined
 -define(DEFAULT_VALIDATE_REQUEST_INFO,        undefined).
 -define(DEFAULT_VALIDATE_REQUEST,             undefined).
 -define(DEFAULT_VALIDATE_RESPONSE_INFO,       undefined).
@@ -178,6 +195,8 @@
     list({retry, non_neg_integer()} |
          {retry_delay, non_neg_integer()} |
          {ordered, boolean()} |
+         {timeout_default, cloudi_service:timeout_milliseconds()} |
+         {priority_default, cloudi_service:priority()} |
          {validate_request_info,
           fun((RequestInfo :: cloudi_service:request_info()) -> boolean()) |
           {Module1 :: module(), Function1 :: atom()}} |
@@ -441,6 +460,8 @@ mcast(Dispatcher, Name, RequestInfo, Request, Timeout, Priority,
     end;
 mcast(Dispatcher, Name, RequestInfo, Request, Timeout, Priority,
       #cloudi_queue{ordered = Ordered,
+                    timeout_default = TimeoutDefault,
+                    priority_default = PriorityDefault,
                     validate_request_info = RequestInfoF,
                     validate_request = RequestF,
                     failures_source_die = FailuresSrcDie,
@@ -453,7 +474,8 @@ mcast(Dispatcher, Name, RequestInfo, Request, Timeout, Priority,
                   RequestInfo, Request) of
         true ->
             case queue_mcast(Dispatcher, Name, RequestInfo, Request,
-                             Timeout, Priority, Requests, Ordered) of
+                             Timeout, Priority, Requests, Ordered,
+                             TimeoutDefault, PriorityDefault) of
                 {ok, OrderedPending, RequestsNew} ->
                     {ok, State#cloudi_queue{ordered_pending = OrderedPending,
                                             requests = RequestsNew}};
@@ -503,6 +525,8 @@ new(Options)
         {retry,                         ?DEFAULT_RETRY},
         {retry_delay,                   ?DEFAULT_RETRY_DELAY},
         {ordered,                       ?DEFAULT_ORDERED},
+        {timeout_default,               ?DEFAULT_TIMEOUT_DEFAULT},
+        {priority_default,              ?DEFAULT_PRIORITY_DEFAULT},
         {validate_request_info,         ?DEFAULT_VALIDATE_REQUEST_INFO},
         {validate_request,              ?DEFAULT_VALIDATE_REQUEST},
         {validate_response_info,        ?DEFAULT_VALIDATE_RESPONSE_INFO},
@@ -510,7 +534,8 @@ new(Options)
         {failures_source_die,           ?DEFAULT_FAILURES_SOURCE_DIE},
         {failures_source_max_count,     ?DEFAULT_FAILURES_SOURCE_MAX_COUNT},
         {failures_source_max_period,    ?DEFAULT_FAILURES_SOURCE_MAX_PERIOD}],
-    [Retry, RetryDelay, Ordered, ValidateRequestInfo0, ValidateRequest0,
+    [Retry, RetryDelay, Ordered, TimeoutDefault, PriorityDefault,
+     ValidateRequestInfo0, ValidateRequest0,
      ValidateResponseInfo0, ValidateResponse0,
      FailuresSrcDie, FailuresSrcMaxCount, FailuresSrcMaxPeriod
      ] =
@@ -519,6 +544,16 @@ new(Options)
     true = is_integer(RetryDelay) andalso
            (RetryDelay >= 0) andalso (RetryDelay =< 4294967295),
     true = is_boolean(Ordered),
+    true = (TimeoutDefault =:= undefined) orelse
+           (TimeoutDefault =:= limit_min) orelse
+           (TimeoutDefault =:= limit_max) orelse
+           (is_integer(TimeoutDefault) andalso
+            (TimeoutDefault >= 0) andalso
+            (TimeoutDefault =< ?TIMEOUT_MAX)),
+    true = (PriorityDefault =:= undefined) orelse
+           (is_integer(PriorityDefault) andalso
+            (PriorityDefault >= ?PRIORITY_HIGH) andalso
+            (PriorityDefault =< ?PRIORITY_LOW)),
     ValidateRequestInfo1 = cloudi_args_type:
                            function_optional(ValidateRequestInfo0, 1),
     ValidateRequest1 = cloudi_args_type:
@@ -537,6 +572,8 @@ new(Options)
         retry = Retry,
         retry_delay = RetryDelay,
         ordered = Ordered,
+        timeout_default = TimeoutDefault,
+        priority_default = PriorityDefault,
         word_size = WordSize,
         validate_request_info = ValidateRequestInfo1,
         validate_request = ValidateRequest1,
@@ -848,6 +885,8 @@ send_id(Dispatcher, Name, RequestInfo, Request, Timeout, Priority, PatternPid,
     end;
 send_id(Dispatcher, Name, RequestInfo, Request, Timeout, Priority, PatternPid,
         #cloudi_queue{ordered = Ordered,
+                      timeout_default = TimeoutDefault,
+                      priority_default = PriorityDefault,
                       validate_request_info = RequestInfoF,
                       validate_request = RequestF,
                       failures_source_die = FailuresSrcDie,
@@ -860,7 +899,8 @@ send_id(Dispatcher, Name, RequestInfo, Request, Timeout, Priority, PatternPid,
                   RequestInfo, Request) of
         true ->
             case queue_send_first(Dispatcher, Name, RequestInfo, Request,
-                                  Timeout, Priority, PatternPid) of
+                                  Timeout, Priority, PatternPid,
+                                  TimeoutDefault, PriorityDefault) of
                 {ok, TransId, RequestState} ->
                     OrderedPending = if
                         Ordered =:= true ->
@@ -999,20 +1039,28 @@ validate(RInfoF, undefined, RInfo, _) ->
 validate(RInfoF, RF, RInfo, R) ->
     validate_f_return(RInfoF(RInfo)) andalso validate_f_return(RF(RInfo, R)).
 
+default(undefined, Default) ->
+    Default;
+default(Value, _) ->
+    Value.
+
 queue_send_first(Dispatcher, Name, RequestInfo, Request,
-                 Timeout, Priority, undefined) ->
-    case cloudi_service:get_pid(Dispatcher, Name, Timeout) of
+                 Timeout, Priority, undefined,
+                 TimeoutDefault, PriorityDefault) ->
+    TimeoutNew = default(Timeout, TimeoutDefault),
+    PriorityNew = default(Priority, PriorityDefault),
+    case cloudi_service:get_pid(Dispatcher, Name, TimeoutNew) of
         {ok, PatternPid} ->
             case cloudi_service:send_async_active(Dispatcher, Name,
                                                   RequestInfo, Request,
-                                                  Timeout, Priority,
+                                                  TimeoutNew, PriorityNew,
                                                   PatternPid) of
                 {ok, TransId} ->
                     RequestState = #request{name = Name,
                                             request_info = RequestInfo,
                                             request = Request,
-                                            timeout = Timeout,
-                                            priority = Priority,
+                                            timeout = TimeoutNew,
+                                            priority = PriorityNew,
                                             id = TransId,
                                             pattern_pid = PatternPid,
                                             retry_pattern_pid = false},
@@ -1024,17 +1072,20 @@ queue_send_first(Dispatcher, Name, RequestInfo, Request,
             Error
     end;
 queue_send_first(Dispatcher, Name, RequestInfo, Request,
-                 Timeout, Priority, PatternPid) ->
+                 Timeout, Priority, PatternPid,
+                 TimeoutDefault, PriorityDefault) ->
+    TimeoutNew = default(Timeout, TimeoutDefault),
+    PriorityNew = default(Priority, PriorityDefault),
     case cloudi_service:send_async_active(Dispatcher, Name,
                                           RequestInfo, Request,
-                                          Timeout, Priority,
+                                          TimeoutNew, PriorityNew,
                                           PatternPid) of
         {ok, TransId} ->
             RequestState = #request{name = Name,
                                     request_info = RequestInfo,
                                     request = Request,
-                                    timeout = Timeout,
-                                    priority = Priority,
+                                    timeout = TimeoutNew,
+                                    priority = PriorityNew,
                                     id = TransId,
                                     pattern_pid = PatternPid,
                                     retry_pattern_pid = true},
@@ -1043,20 +1094,23 @@ queue_send_first(Dispatcher, Name, RequestInfo, Request,
             Error
     end.
 
-queue_send_first(Dispatcher, Name, RequestInfo, Request,
-                 Timeout, Priority, TransId, undefined) ->
-    case cloudi_service:get_pid(Dispatcher, Name, Timeout) of
+queue_send_first_ordered(Dispatcher, Name, RequestInfo, Request,
+                         Timeout, Priority, TransId, undefined,
+                         TimeoutDefault, PriorityDefault) ->
+    TimeoutNew = default(Timeout, TimeoutDefault),
+    PriorityNew = default(Priority, PriorityDefault),
+    case cloudi_service:get_pid(Dispatcher, Name, TimeoutNew) of
         {ok, PatternPid} ->
             case cloudi_service:send_async_active(Dispatcher, Name,
                                                   RequestInfo, Request,
-                                                  Timeout, Priority,
+                                                  TimeoutNew, PriorityNew,
                                                   TransId, PatternPid) of
                 {ok, TransId} ->
                     RequestState = #request{name = Name,
                                             request_info = RequestInfo,
                                             request = Request,
-                                            timeout = Timeout,
-                                            priority = Priority,
+                                            timeout = TimeoutNew,
+                                            priority = PriorityNew,
                                             id = TransId,
                                             pattern_pid = PatternPid,
                                             retry_pattern_pid = false},
@@ -1067,18 +1121,21 @@ queue_send_first(Dispatcher, Name, RequestInfo, Request,
         {error, _} = Error ->
             Error
     end;
-queue_send_first(Dispatcher, Name, RequestInfo, Request,
-                 Timeout, Priority, TransId, PatternPid) ->
+queue_send_first_ordered(Dispatcher, Name, RequestInfo, Request,
+                         Timeout, Priority, TransId, PatternPid,
+                         TimeoutDefault, PriorityDefault) ->
+    TimeoutNew = default(Timeout, TimeoutDefault),
+    PriorityNew = default(Priority, PriorityDefault),
     case cloudi_service:send_async_active(Dispatcher, Name,
                                           RequestInfo, Request,
-                                          Timeout, Priority,
+                                          TimeoutNew, PriorityNew,
                                           TransId, PatternPid) of
         {ok, TransId} ->
             RequestState = #request{name = Name,
                                     request_info = RequestInfo,
                                     request = Request,
-                                    timeout = Timeout,
-                                    priority = Priority,
+                                    timeout = TimeoutNew,
+                                    priority = PriorityNew,
                                     id = TransId,
                                     pattern_pid = PatternPid,
                                     retry_pattern_pid = true},
@@ -1129,21 +1186,24 @@ queue_send_retry(Dispatcher,
     end.
 
 queue_mcast(Dispatcher, Name, RequestInfo, Request,
-            Timeout, Priority, Requests, Ordered) ->
-    case cloudi_service:get_pids(Dispatcher, Name, Timeout) of
+            Timeout, Priority, Requests, Ordered,
+            TimeoutDefault, PriorityDefault) ->
+    TimeoutNew = default(Timeout, TimeoutDefault),
+    PriorityNew = default(Priority, PriorityDefault),
+    case cloudi_service:get_pids(Dispatcher, Name, TimeoutNew) of
         {ok, PatternPids} ->
-            queue_mcast(PatternPids, 0,
-                        Dispatcher, Name, RequestInfo, Request,
-                        Timeout, Priority, Requests, Ordered);
+            queue_mcast_send(PatternPids, 0,
+                             Dispatcher, Name, RequestInfo, Request,
+                             TimeoutNew, PriorityNew, Requests, Ordered);
         {error, _} = Error ->
             {Error, 0, Requests}
     end.
 
-queue_mcast([], OrderedPending, _, _, _, _, _, _, Requests, _) ->
+queue_mcast_send([], OrderedPending, _, _, _, _, _, _, Requests, _) ->
     {ok, OrderedPending, Requests};
-queue_mcast([PatternPid | PatternPids], OrderedPending,
-            Dispatcher, Name, RequestInfo, Request,
-            Timeout, Priority, Requests, Ordered) ->
+queue_mcast_send([PatternPid | PatternPids], OrderedPending,
+                 Dispatcher, Name, RequestInfo, Request,
+                 Timeout, Priority, Requests, Ordered) ->
     case cloudi_service:send_async_active(Dispatcher, Name,
                                           RequestInfo, Request,
                                           Timeout, Priority,
@@ -1163,12 +1223,12 @@ queue_mcast([PatternPid | PatternPids], OrderedPending,
                 Ordered =:= false ->
                     OrderedPending
             end,
-            queue_mcast(PatternPids, OrderedPendingNew,
-                        Dispatcher, Name, RequestInfo, Request,
-                        Timeout, Priority,
-                        maps:put(TransId,
-                                 RequestState,
-                                 Requests), Ordered);
+            queue_mcast_send(PatternPids, OrderedPendingNew,
+                             Dispatcher, Name, RequestInfo, Request,
+                             Timeout, Priority,
+                             maps:put(TransId,
+                                      RequestState,
+                                      Requests), Ordered);
         {error, _} = Error ->
             {Error, OrderedPending, Requests}
     end.
@@ -1222,14 +1282,17 @@ ordered_send(Dispatcher,
                                    id = TransId,
                                    pattern_pid = PatternPid},
              #cloudi_queue{ordered = true,
+                           timeout_default = TimeoutDefault,
+                           priority_default = PriorityDefault,
                            failures_source_die = FailuresSrcDie,
                            failures_source_max_count = FailuresSrcMaxCount,
                            failures_source_max_period = FailuresSrcMaxPeriod,
                            failures_source = FailuresSrc,
                            ordered_pending = 0,
                            requests = Requests} = State) ->
-    case queue_send_first(Dispatcher, Name, RequestInfo, Request,
-                          Timeout, Priority, TransId, PatternPid) of
+    case queue_send_first_ordered(Dispatcher, Name, RequestInfo, Request,
+                                  Timeout, Priority, TransId, PatternPid,
+                                  TimeoutDefault, PriorityDefault) of
         {ok, RequestState} ->
             {ok,
              State#cloudi_queue{ordered_pending = 1,
@@ -1252,6 +1315,8 @@ ordered_mcast(Dispatcher,
                                      timeout = Timeout,
                                      priority = Priority},
               #cloudi_queue{ordered = true = Ordered,
+                            timeout_default = TimeoutDefault,
+                            priority_default = PriorityDefault,
                             failures_source_die = FailuresSrcDie,
                             failures_source_max_count = FailuresSrcMaxCount,
                             failures_source_max_period = FailuresSrcMaxPeriod,
@@ -1259,7 +1324,8 @@ ordered_mcast(Dispatcher,
                             ordered_pending = 0,
                             requests = Requests} = State) ->
     case queue_mcast(Dispatcher, Name, RequestInfo, Request,
-                     Timeout, Priority, Requests, Ordered) of
+                     Timeout, Priority, Requests, Ordered,
+                     TimeoutDefault, PriorityDefault) of
         {ok, OrderedPending, RequestsNew} ->
             {ok, State#cloudi_queue{ordered_pending = OrderedPending,
                                     requests = RequestsNew}};
