@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,13 +29,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([acl_add/2,
@@ -45,8 +45,9 @@
          services_add/2,
          services_remove/2,
          services_restart/2,
-         services_update/2,
          services_search/2,
+         services_status/2,
+         services_update/2,
          services/1,
          nodes_set/2,
          nodes_get/1,
@@ -57,15 +58,16 @@
          nodes/1,
          logging_set/2,
          logging_file_set/2,
-         logging_stdout_set/2,
          logging_level_set/2,
+         logging_stdout_set/2,
          logging_syslog_set/2,
          logging_formatters_set/2,
          logging_redirect_set/2,
          logging/1,
          code_path_add/2,
          code_path_remove/2,
-         code_path/1]).
+         code_path/1,
+         code_status/1]).
 
 -include("cloudi_service_api.hrl").
 -include("cloudi_core_i_constants.hrl").
@@ -547,11 +549,67 @@
                   {options, service_options_internal() |
                             service_options_external()}).
 -type service() :: #internal{} | #external{}.
+-type service_status_internal() ::
+    nonempty_list({count_process, pos_integer()} |
+                  {uptime_total, nonempty_string()} |
+                  {uptime_running, nonempty_string()} |
+                  {uptime_restarts, nonempty_string()} |
+                  {downtime_day_restarting, nonempty_string()} |
+                  {downtime_week_restarting, nonempty_string()} |
+                  {downtime_month_restarting, nonempty_string()} |
+                  {downtime_year_restarting, nonempty_string()} |
+                  {interrupt_day_updating, nonempty_string()} |
+                  {interrupt_week_updating, nonempty_string()} |
+                  {interrupt_month_updating, nonempty_string()} |
+                  {interrupt_year_updating, nonempty_string()} |
+                  {availability_day_total, nonempty_string()} |
+                  {availability_day_running, nonempty_string()} |
+                  {availability_day_updated, nonempty_string()} |
+                  {availability_week_total, nonempty_string()} |
+                  {availability_week_running, nonempty_string()} |
+                  {availability_week_updated, nonempty_string()} |
+                  {availability_month_total, nonempty_string()} |
+                  {availability_month_running, nonempty_string()} |
+                  {availability_month_updated, nonempty_string()} |
+                  {availability_year_total, nonempty_string()} |
+                  {availability_year_running, nonempty_string()} |
+                  {availability_year_updated, nonempty_string()}).
+-type service_status_external() ::
+    nonempty_list({count_process, pos_integer()} |
+                  {count_thread, pos_integer()} |
+                  {uptime_total, nonempty_string()} |
+                  {uptime_running, nonempty_string()} |
+                  {uptime_restarts, nonempty_string()} |
+                  {downtime_day_restarting, nonempty_string()} |
+                  {downtime_week_restarting, nonempty_string()} |
+                  {downtime_month_restarting, nonempty_string()} |
+                  {downtime_year_restarting, nonempty_string()} |
+                  {interrupt_day_updating, nonempty_string()} |
+                  {interrupt_week_updating, nonempty_string()} |
+                  {interrupt_month_updating, nonempty_string()} |
+                  {interrupt_year_updating, nonempty_string()} |
+                  {availability_day_total, nonempty_string()} |
+                  {availability_day_running, nonempty_string()} |
+                  {availability_day_updated, nonempty_string()} |
+                  {availability_week_total, nonempty_string()} |
+                  {availability_week_running, nonempty_string()} |
+                  {availability_week_updated, nonempty_string()} |
+                  {availability_month_total, nonempty_string()} |
+                  {availability_month_running, nonempty_string()} |
+                  {availability_month_updated, nonempty_string()} |
+                  {availability_year_total, nonempty_string()} |
+                  {availability_year_running, nonempty_string()} |
+                  {availability_year_updated, nonempty_string()}).
+-type service_status() ::
+    service_status_internal() | service_status_external().
 -export_type([service_id/0,
               service_internal/0,
               service_external/0,
+              service_proplist/0,
               service/0,
-              service_proplist/0]).
+              service_status_internal/0,
+              service_status_external/0,
+              service_status/0]).
 
 -type module_version() :: list(any()).
 -type module_state_internal_f() ::
@@ -761,7 +819,7 @@
 -type loglevel() :: loglevel_on() | off.
 -type loglevel_on() :: fatal | error | warn | info | debug | trace.
 -type logging_syslog_identity() :: nonempty_string().
--type logging_syslog_facility() :: 
+-type logging_syslog_facility() ::
     kernel | user | mail | daemon | auth0 | syslog |
     print | news | uucp | clock0 | auth1 | ftp | ntp |
     auth2 | auth3 | clock1 | local0 | local1 | local2 |
@@ -803,6 +861,7 @@
                   {redirect, node() | undefined} |
                   {syslog, logging_syslog_set_proplist() | undefined} |
                   {formatters, logging_formatters_set_proplist() | undefined} |
+                  {log_time_offset, loglevel()} |
                   {aspects_log_before, list(aspect_log_before())} |
                   {aspects_log_after, list(aspect_log_after())}).
 -export_type([loglevel/0,
@@ -819,6 +878,42 @@
               logging_formatters_set_proplist/0,
               logging_proplist/0]).
 
+-type code_status() ::
+    nonempty_list({build_machine, nonempty_string()} |
+                  {build_kernel_version, nonempty_string()} |
+                  {build_operating_system, nonempty_string()} |
+                  {build_erlang_otp_release, nonempty_string()} |
+                  {build_cloudi_time, nonempty_string()} |
+                  {build_cloudi_cxx_compiler_version, nonempty_string()} |
+                  {build_cloudi_cxx_dependencies_versions, nonempty_string()} |
+                  {build_erts_c_compiler_version, nonempty_string()} |
+                  {install_erlang_erts_time, nonempty_string()} |
+                  {install_erlang_kernel_time, nonempty_string()} |
+                  {install_erlang_stdlib_time, nonempty_string()} |
+                  {install_erlang_sasl_time, nonempty_string()} |
+                  {install_erlang_compiler_time, nonempty_string()} |
+                  {install_cloudi_time, nonempty_string()} |
+                  {runtime_erlang_erts_version, nonempty_string()} |
+                  {runtime_erlang_kernel_version, nonempty_string()} |
+                  {runtime_erlang_stdlib_version, nonempty_string()} |
+                  {runtime_erlang_sasl_version, nonempty_string()} |
+                  {runtime_erlang_compiler_version, nonempty_string()} |
+                  {runtime_cloudi_version, nonempty_string()} |
+                  {runtime_machine_processors, pos_integer()} |
+                  {runtime_start, nonempty_string()} |
+                  {runtime_clock, nonempty_string()} |
+                  {runtime_clock_offset, nonempty_string()} |
+                  {runtime_total, nonempty_string()} |
+                  {runtime_cloudi_start, nonempty_string()} |
+                  {runtime_cloudi_total, nonempty_string()} |
+                  {runtime_cloudi_changes,
+                   list(nonempty_list({type, internal | external} |
+                                      {file_age, nonempty_string()} |
+                                      {file_path, nonempty_string()} |
+                                      {service_ids,
+                                       nonempty_list(service_id())}))}).
+-export_type([code_status/0]).
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -840,7 +935,7 @@
 %% @end
 %%-------------------------------------------------------------------------
 
--spec acl_add(L :: nonempty_list({atom(), acl()}), 
+-spec acl_add(L :: nonempty_list({atom(), acl()}),
               Timeout :: api_timeout_milliseconds()) ->
     ok |
     {error,
@@ -997,7 +1092,7 @@ services_remove([_ | _] = L, Timeout)
 %% @end
 %%-------------------------------------------------------------------------
 
--spec services_restart(L :: nonempty_list(binary() | string()), 
+-spec services_restart(L :: nonempty_list(binary() | string()),
                        Timeout :: api_timeout_milliseconds()) ->
     ok |
     {error,
@@ -1018,6 +1113,83 @@ services_restart([_ | _] = L, Timeout)
             Error
     end.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Search service instances for matches on the provided service name.===
+%% Multiple services may be returned for a single service name.  Only service
+%% instances on the local Erlang node are searched.  Service names that match
+%% subscriptions of non-service Erlang pids only
+%% (e.g., cloudi_service_http_cowboy websocket connection pids) will not
+%% return the service's configuration with this function.  Provide a scope
+%% within a 2 element tuple with the service name to check a custom scope.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec services_search(Name :: {atom(), cloudi:service_name()} |
+                              cloudi:service_name(),
+                      Timeout :: api_timeout_milliseconds()) ->
+    {ok, list({service_id(), service_internal()} |
+              {service_id(), service_external()})} |
+    {error,
+     timeout | noproc |
+     service_name_invalid |
+     cloudi_core_i_configuration:error_reason_services_search()}.
+
+services_search(Name, Timeout)
+    when ((is_integer(Timeout) andalso
+           (Timeout > ?TIMEOUT_DELTA) andalso
+           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+          (Timeout =:= infinity)) ->
+    {Scope, ServiceName} = case Name of
+        {ScopeValue, [_ | _] = ServiceNameValue} when is_atom(ScopeValue) ->
+            {?SCOPE_ASSIGN(ScopeValue), ServiceNameValue};
+        [_ | _] = ServiceNameValue ->
+            {?SCOPE_DEFAULT, ServiceNameValue}
+    end,
+    try cloudi_x_trie:is_pattern(ServiceName) of
+        false ->
+            cloudi_core_i_configurator:services_search(Scope, ServiceName,
+                                                       Timeout);
+        true ->
+            {error, service_name_invalid}
+    catch
+        exit:badarg ->
+            {error, service_name_invalid}
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===List the current status of specific services.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec services_status(L :: list(binary() | string()),
+                      Timeout :: api_timeout_milliseconds()) ->
+    {ok, nonempty_list({service_id(), service_status()})} |
+    {error,
+     timeout | noproc |
+     {service_id_invalid, any()} |
+     {service_not_found, any()}}.
+
+services_status(L, Timeout)
+    when is_list(L),
+         ((is_integer(Timeout) andalso
+           (Timeout > ?TIMEOUT_DELTA) andalso
+           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+          (Timeout =:= infinity)) ->
+    ServiceIdsResult = case service_ids_convert(L) of
+        {ok, []} ->
+            cloudi_core_i_configurator:service_ids(Timeout);
+        ServiceIdsResultValue ->
+            ServiceIdsResultValue
+    end,
+    case ServiceIdsResult of
+        {ok, ServiceIdsValid} ->
+            cloudi_core_i_services_monitor:status(ServiceIdsValid, Timeout);
+        {error, _} = Error ->
+            Error
+    end.
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Update service instances.===
@@ -1052,49 +1224,6 @@ services_update([_ | _] = L, Timeout)
             Error
     end.
 
-%%-------------------------------------------------------------------------
-%% @doc
-%% ===Search service instances for matches on the provided service name.===
-%% Multiple services may be returned for a single service name.  Only service
-%% instances on the local Erlang node are searched.  Service names that match
-%% subscriptions of non-service Erlang pids only
-%% (e.g., cloudi_service_http_cowboy websocket connection pids) will not
-%% return the service's configuration with this function.  Provide a scope
-%% within a 2 element tuple with the service name to check a custom scope.
-%% @end
-%%-------------------------------------------------------------------------
-
--spec services_search(Name :: {atom(), cloudi:service_name()} |
-                              cloudi:service_name(),
-                      Timeout :: api_timeout_milliseconds()) ->
-    {ok, list({service_id(), service_internal()} |
-              {service_id(), service_external()})} |
-    {error,
-     timeout | noproc |
-     service_name_invalid}.
-
-services_search(Name, Timeout)
-    when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
-          (Timeout =:= infinity)) ->
-    {Scope, ServiceName} = case Name of
-        {ScopeValue, [_ | _] = ServiceNameValue} when is_atom(ScopeValue) ->
-            {?SCOPE_ASSIGN(ScopeValue), ServiceNameValue};
-        [_ | _] = ServiceNameValue ->
-            {?SCOPE_DEFAULT, ServiceNameValue}
-    end,
-    try cloudi_x_trie:is_pattern(ServiceName) of
-        false ->
-            cloudi_core_i_configurator:services_search(Scope, ServiceName,
-                                                       Timeout);
-        true ->
-            {error, service_name_invalid}
-    catch
-        exit:badarg ->
-            {error, service_name_invalid}
-    end.
-
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===List all service instances with each service's UUID.===
@@ -1280,24 +1409,6 @@ logging_file_set(FilePath, Timeout)
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_file_set(FilePath, Timeout).
 
-%%-------------------------------------------------------------------------
-%% @doc
-%% ===Modify the current log stdout usage.===
-%% @end
-%%-------------------------------------------------------------------------
-
--spec logging_stdout_set(Stdout :: boolean(),
-                         Timeout :: api_timeout_milliseconds()) ->
-    ok.
-
-logging_stdout_set(Stdout, Timeout)
-    when is_boolean(Stdout),
-         ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
-          (Timeout =:= infinity)) ->
-    cloudi_core_i_configurator:logging_stdout_set(Stdout, Timeout).
-
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Modify the current loglevel.===
@@ -1321,13 +1432,32 @@ logging_level_set(Level, Timeout)
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_level_set(Level, Timeout).
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Modify the current log stdout usage.===
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec logging_stdout_set(Stdout :: boolean(),
+                         Timeout :: api_timeout_milliseconds()) ->
+    ok.
+
+logging_stdout_set(Stdout, Timeout)
+    when is_boolean(Stdout),
+         ((is_integer(Timeout) andalso
+           (Timeout > ?TIMEOUT_DELTA) andalso
+           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+          (Timeout =:= infinity)) ->
+    cloudi_core_i_configurator:logging_stdout_set(Stdout, Timeout).
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Set the CloudI syslog configuration.===
 %% @end
 %%-------------------------------------------------------------------------
 
--spec logging_syslog_set(L :: logging_syslog_set_proplist(),
+-spec logging_syslog_set(L :: logging_syslog_set_proplist() |
+                              undefined,
                          Timeout :: api_timeout_milliseconds()) ->
     ok |
     {error,
@@ -1335,7 +1465,7 @@ logging_level_set(Level, Timeout)
      cloudi_core_i_configuration:error_reason_logging_syslog_set()}.
 
 logging_syslog_set(L, Timeout)
-    when is_list(L),
+    when is_list(L) orelse (L =:= undefined),
          ((is_integer(Timeout) andalso
            (Timeout > ?TIMEOUT_DELTA) andalso
            (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
@@ -1351,7 +1481,8 @@ logging_syslog_set(L, Timeout)
 %% @end
 %%-------------------------------------------------------------------------
 
--spec logging_formatters_set(L :: logging_formatters_set_proplist(),
+-spec logging_formatters_set(L :: logging_formatters_set_proplist() |
+                                  undefined,
                              Timeout :: api_timeout_milliseconds()) ->
     ok |
     {error,
@@ -1359,7 +1490,7 @@ logging_syslog_set(L, Timeout)
      cloudi_core_i_configuration:error_reason_logging_formatters_set()}.
 
 logging_formatters_set(L, Timeout)
-    when is_list(L),
+    when is_list(L) orelse (L =:= undefined),
          ((is_integer(Timeout) andalso
            (Timeout > ?TIMEOUT_DELTA) andalso
            (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
@@ -1432,7 +1563,7 @@ code_path_add(Dir, Timeout)
 %% @doc
 %% ===Remove a directory from the CloudI Erlang VM code server's search paths.===
 %% This doesn't impact any running services, only services that will be
-%% started in the future. 
+%% started in the future.
 %% @end
 %%-------------------------------------------------------------------------
 
@@ -1472,6 +1603,77 @@ code_path(Timeout)
           (Timeout =:= infinity)) ->
     {ok, code:get_path()}.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Provide the current CloudI Erlang VM code status.===
+%% Both build and runtime information is provided with the
+%% service files changed after CloudI was started
+%% (probably due to service updates).
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec code_status(Timeout :: api_timeout_milliseconds()) ->
+    {ok, code_status()} |
+    {error,
+     timeout | noproc |
+     cloudi_core_i_configuration:error_reason_code_status()}.
+
+code_status(Timeout)
+    when ((is_integer(Timeout) andalso
+           (Timeout > ?TIMEOUT_DELTA) andalso
+           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+          (Timeout =:= infinity)) ->
+    TimeNative = cloudi_timestamp:native_monotonic(),
+    TimeOffset = erlang:time_offset(),
+    TimeOS = cloudi_timestamp:native_os(),
+    TimeNativeStart = erlang:system_info(start_time),
+    TimeNativeTotal = TimeNative - TimeNativeStart,
+    TimeNativeNow = TimeNative + TimeOffset,
+    case cloudi_core_i_configurator:code_status(TimeNative, TimeOffset,
+                                                Timeout) of
+        {ok, TimeNativeConfig, RuntimeChanges} ->
+            MicroSecondsStart = cloudi_timestamp:
+                                convert(TimeNativeStart + TimeOffset,
+                                        native, microsecond),
+            MicroSecondsNow = cloudi_timestamp:
+                              convert(TimeNativeNow, native, microsecond),
+            NanoSecondsOffset = cloudi_timestamp:
+                                convert(TimeOS - TimeNativeNow,
+                                        native, nanosecond),
+            NanoSecondsTotal = cloudi_timestamp:
+                               convert(TimeNativeTotal, native, nanosecond),
+            MicroSecondsStartConfig = cloudi_timestamp:
+                                      convert(TimeNativeConfig + TimeOffset,
+                                              native, microsecond),
+            NanoSecondsTotalConfig = cloudi_timestamp:
+                                     convert(TimeNative - TimeNativeConfig,
+                                             native, nanosecond),
+            Status = cloudi_environment:status() ++
+                [{runtime_start,
+                  cloudi_timestamp:
+                  microseconds_epoch_to_string(MicroSecondsStart)},
+                 {runtime_clock,
+                  cloudi_timestamp:
+                  microseconds_epoch_to_string(MicroSecondsNow)},
+                 {runtime_clock_offset,
+                  cloudi_timestamp:
+                  nanoseconds_to_string(NanoSecondsOffset, signed)},
+                 {runtime_total,
+                  cloudi_timestamp:
+                  nanoseconds_to_string(NanoSecondsTotal)},
+                 {runtime_cloudi_start,
+                  cloudi_timestamp:
+                  microseconds_epoch_to_string(MicroSecondsStartConfig)},
+                 {runtime_cloudi_total,
+                  cloudi_timestamp:
+                  nanoseconds_to_string(NanoSecondsTotalConfig)},
+                 {runtime_cloudi_changes,
+                  RuntimeChanges}],
+            {ok, Status};
+        {error, _} = Error ->
+            Error
+    end.
+
 %%%------------------------------------------------------------------------
 %%% Private functions
 %%%------------------------------------------------------------------------
