@@ -5,45 +5,31 @@
 %%% ==CloudI Core Basic Tests==
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2014-2017, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2017 Michael Truog
-%%% @version 1.5.5 {@date} {@time}
+%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
+%%%
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2014-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_SUITE).
@@ -68,7 +54,8 @@
          end_per_testcase/2]).
 
 %% test callbacks
--export([t_service_internal_sync_1/1,
+-export([t_cloudi_code_status_1/1,
+         t_service_internal_sync_1/1,
          t_service_internal_sync_2/1,
          t_service_internal_sync_3/1,
          t_service_internal_async_1/1,
@@ -233,10 +220,9 @@ cloudi_service_handle_info(increment,
                            #state{count = Count} = State, _Dispatcher) ->
     {noreply, State#state{count = Count + 2}};
 cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    {stop, {unexpected_info, Request}, State}.
+    {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-cloudi_service_terminate(_Reason, _Timeout,
-                         undefined) ->
+cloudi_service_terminate(_Reason, _Timeout, undefined) ->
     % cloudi_service_init/3 caused an exception
     ok;
 cloudi_service_terminate(terminate_sleep_test_1, _Timeout,
@@ -268,11 +254,14 @@ cloudi_service_terminate(_Reason, _Timeout,
 %%%------------------------------------------------------------------------
 
 all() ->
-    [{group, service_internal_1},
+    [{group, cloudi_runtime_1},
+     {group, service_internal_1},
      {group, cloudi_modules_1}].
 
 groups() ->
-    [{service_internal_1, [],
+    [{cloudi_runtime_1, [sequence],
+      [t_cloudi_code_status_1]},
+     {service_internal_1, [sequence],
       [t_service_internal_sync_1,
        t_service_internal_sync_2,
        t_service_internal_sync_3,
@@ -286,7 +275,7 @@ groups() ->
        t_service_internal_terminate_4,
        t_service_internal_update_1,
        t_service_internal_log_1]},
-     {cloudi_modules_1, [],
+     {cloudi_modules_1, [parallel],
       [t_cloudi_args_type_1,
        t_cloudi_service_name_1]}].
 
@@ -316,12 +305,22 @@ end_per_group(_GroupName, Config) ->
 
 init_per_testcase(TestCase) ->
     error_logger:info_msg("~p init~n", [TestCase]),
-    error_logger:tty(false),
+    if
+        TestCase =:= t_cloudi_code_status_1 ->
+            ok;
+        true ->
+            error_logger:tty(false)
+    end,
     ?LOG_INFO("~p init", [TestCase]),
     ok.
 
 end_per_testcase(TestCase) ->
-    error_logger:tty(true),
+    if
+        TestCase =:= t_cloudi_code_status_1 ->
+            ok;
+        true ->
+            error_logger:tty(true)
+    end,
     error_logger:info_msg("~p end~n", [TestCase]),
     ?LOG_INFO("~p end", [TestCase]),
     ok.
@@ -497,6 +496,12 @@ end_per_testcase(TestCase, Config) ->
 %%% test cases
 %%%------------------------------------------------------------------------
 
+t_cloudi_code_status_1(_Config) ->
+    {ok, CodeStatus} = cloudi_service_api:code_status(infinity),
+    error_logger:info_msg("~p~n", [CodeStatus]),
+    ?LOG_INFO("~p", [CodeStatus]),
+    ok.
+
 t_service_internal_sync_1(_Config) ->
     % make sure synchronous sends work normally,
     % fail within the cloudi_service_init/3 callback with invalid_state,
@@ -519,9 +524,7 @@ t_service_internal_sync_1(_Config) ->
     true = cloudi_x_uuid:is_v1(TransId1),
     {error,
      {service_internal_start_failed,
-      {error,
-       {{badmatch, {error, invalid_state}},
-        _}}}} = cloudi_service_api:services_add([
+      {error, invalid_state}}} = cloudi_service_api:services_add([
         [{prefix, ?SERVICE_PREFIX1},
          {module, ?MODULE},
          {args, [{mode, init_send_sync}]},
@@ -542,9 +545,7 @@ t_service_internal_sync_1(_Config) ->
     true = (TransId2 > TransId1),
     {error,
      {service_internal_start_failed,
-      {error,
-       {{badmatch, {error, invalid_state}},
-        _}}}} = cloudi_service_api:services_add([
+      {error, invalid_state}}} = cloudi_service_api:services_add([
         [{prefix, ?SERVICE_PREFIX1},
          {module, ?MODULE},
          {args, [{mode, init_send_sync}]},
@@ -604,12 +605,11 @@ t_service_internal_async_1(_Config) ->
     true = cloudi_x_uuid:is_v1(TransId1),
     {error,
      {service_internal_start_failed,
-      {error,
-       {{badmatch, {error, invalid_state}},
-        _}}}} = cloudi_service_api:services_add([
+      {error, invalid_state}}} = cloudi_service_api:services_add([
         [{prefix, ?SERVICE_PREFIX1},
          {module, ?MODULE},
          {args, [{mode, init_send_async_recv}]},
+         {max_r, 0},
          {options, [{automatic_loading, false}]}]
         ], infinity),
     {{ok, TransId2},
@@ -630,12 +630,11 @@ t_service_internal_async_1(_Config) ->
     true = (TransId2 > TransId1),
     {error,
      {service_internal_start_failed,
-      {error,
-       {{badmatch, {error, invalid_state}},
-        _}}}} = cloudi_service_api:services_add([
+      {error, invalid_state}}} = cloudi_service_api:services_add([
         [{prefix, ?SERVICE_PREFIX1},
          {module, ?MODULE},
          {args, [{mode, init_send_async_recv}]},
+         {max_r, 0},
          {options, [{automatic_loading, false}]}]
         ], infinity),
     ok.
