@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2017 Michael Truog
-%%% @version 1.7.2 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2014-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_client).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -71,7 +71,6 @@
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
          cloudi_service_handle_request/11,
-         cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
@@ -662,7 +661,7 @@ cloudi_service_handle_request(_Type, Name, _Pattern, RequestInfo, Request,
                                      prefix_length = PrefixLength} = State,
                               _Dispatcher) ->
     RequestStartMicroSec = client_debug_start(DebugLevel),
-    [$/ | Method] = string:to_upper(lists:nthtail(PrefixLength, Name)),
+    [$/ | Method] = cloudi_string:uppercase(lists:nthtail(PrefixLength, Name)),
     HeadersIncoming = headers_request(RequestInfo, InputType),
     {HttpCode,
      HeadersOutgoing,
@@ -674,11 +673,7 @@ cloudi_service_handle_request(_Type, Name, _Pattern, RequestInfo, Request,
                      HeadersOutgoing, Response, RequestStartMicroSec),
     {reply, HeadersOutgoing, Response, State}.
 
-cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
-
-cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
+cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
 
 %%%------------------------------------------------------------------------
@@ -723,7 +718,7 @@ headers_request_filter_host(Host) ->
         [HostName] ->
             {HostName, <<"80">>};
         [HostName, PortStr] ->
-            {HostName, strip_whitespace(PortStr)}
+            {HostName, cloudi_string:trim(PortStr)}
     end.
 
 headers_request_filter(Headers0) ->
@@ -883,6 +878,3 @@ result({{ok, Response}, NewAgent}) ->
 result({{ok, _, _}, _} = Success) ->
     Success.
 
-strip_whitespace(String) when is_binary(String) ->
-    erlang:list_to_binary(string:strip(erlang:binary_to_list(String))).
-
