@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2012-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2012-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2012-2017 Michael Truog
-%%% @version 1.7.2 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2012-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_cowboy_handler).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %-behaviour(cloudi_x_cowboy_http_handler).
 %-behaviour(cloudi_x_cowboy_websocket_handler).
@@ -104,7 +104,7 @@ init(_Transport, Req0,
                     {ok, Req1, State}
             end;
         {Upgrade, Req1} ->
-            ?LOG_ERROR("Unknown protocol: ~p", [Upgrade]),
+            ?LOG_ERROR("Unknown protocol: ~w", [Upgrade]),
             {shutdown, Req1, State}
     end;
 init(_Transport, Req,
@@ -208,7 +208,7 @@ handle(Req0,
                 true ->
                     % handle custom methods, if they occur
                     NameIncoming ++ [$/ |
-                        string:to_lower(erlang:binary_to_list(Method))]
+                        cloudi_string:lowercase(erlang:binary_to_list(Method))]
             end,
             PeerShort = erlang:list_to_binary(inet_parse:ntoa(ClientIpAddr)),
             PeerLong = cloudi_ip_address:to_binary(ClientIpAddr),
