@@ -11,7 +11,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,13 +31,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2015-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2015-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_rest).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -190,7 +190,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
             is_list(Method), is_integer(hd(Method)) ->
                 Method
         end,
-        MethodString = string:to_upper(MethodString),
+        MethodString = cloudi_string:uppercase(MethodString),
         true = is_list(Path) andalso is_integer(hd(Path))
     end, Handlers0),
     InfoN = cloudi_args_type:function_optional(Info0, 3),
@@ -256,8 +256,7 @@ cloudi_service_init(Args, Prefix, Timeout, Dispatcher) ->
         UseOptionsMethod =:= true ->
             lists:map(fun({Path, MethodList1}) ->
                 MethodListN = lists:umerge(MethodList1, ["OPTIONS"]),
-                Methods = erlang:list_to_binary(string:join(MethodListN,
-                                                            ", ")),
+                Methods = erlang:list_to_binary(lists:join(", ", MethodListN)),
                 HandlerOptions = fun(_, _, _, _, _, _, _, _, _,
                                      OptionsHandlerState, _) ->
                     % content-type is set automatically by HTTP process
@@ -409,7 +408,7 @@ cloudi_service_handle_info(Request,
                            Dispatcher) ->
     if
         InfoF =:= undefined ->
-            ?LOG_WARN("Unknown info \"~p\"", [Request]),
+            ?LOG_WARN("Unknown info \"~w\"", [Request]),
             {noreply, State};
         true ->
             case InfoF(Request, StateAPI, Dispatcher) of
@@ -420,6 +419,8 @@ cloudi_service_handle_info(Request,
             end
     end.
 
+cloudi_service_terminate(_Reason, _Timeout, undefined) ->
+    ok;
 cloudi_service_terminate(Reason, Timeout,
                          #state{terminate_f = TerminateF,
                                 api_state = StateAPI}) ->
@@ -466,7 +467,7 @@ subscribe_paths('CONNECT', Path, Formats, API, Lookup, Dispatcher) ->
     subscribe_path(Formats, Path, "/connect", API, Lookup, Dispatcher);
 subscribe_paths(Method, Path, Formats, API, Lookup, Dispatcher)
     when is_list(Method) ->
-    MethodSuffix = [$/ | string:to_lower(Method)],
+    MethodSuffix = [$/ | cloudi_string:lowercase(Method)],
     subscribe_path(Formats, Path, MethodSuffix, API, Lookup, Dispatcher).
 
 response_info_headers(ResponseInfo0, Name, Format,
