@@ -8,49 +8,35 @@
 %%% are useful for runtime usage of applications and releases.
 %%% @end
 %%%
-%%% BSD LICENSE
-%%% 
-%%% Copyright (c) 2013-2016, Michael Truog <mjtruog at gmail dot com>
-%%% All rights reserved.
-%%% 
-%%% Redistribution and use in source and binary forms, with or without
-%%% modification, are permitted provided that the following conditions are met:
-%%% 
-%%%     * Redistributions of source code must retain the above copyright
-%%%       notice, this list of conditions and the following disclaimer.
-%%%     * Redistributions in binary form must reproduce the above copyright
-%%%       notice, this list of conditions and the following disclaimer in
-%%%       the documentation and/or other materials provided with the
-%%%       distribution.
-%%%     * All advertising materials mentioning features or use of this
-%%%       software must display the following acknowledgment:
-%%%         This product includes software developed by Michael Truog
-%%%     * The name of the author may not be used to endorse or promote
-%%%       products derived from this software without specific prior
-%%%       written permission
-%%% 
-%%% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-%%% CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-%%% INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-%%% OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-%%% DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-%%% CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-%%% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-%%% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-%%% SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-%%% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-%%% WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-%%% NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-%%% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-%%% DAMAGE.
+%%% MIT License
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2013-2016 Michael Truog
-%%% @version 1.5.2 {@date} {@time}
+%%% Copyright (c) 2013-2017 Michael Truog <mjtruog at protonmail dot com>
+%%%
+%%% Permission is hereby granted, free of charge, to any person obtaining a
+%%% copy of this software and associated documentation files (the "Software"),
+%%% to deal in the Software without restriction, including without limitation
+%%% the rights to use, copy, modify, merge, publish, distribute, sublicense,
+%%% and/or sell copies of the Software, and to permit persons to whom the
+%%% Software is furnished to do so, subject to the following conditions:
+%%%
+%%% The above copyright notice and this permission notice shall be included in
+%%% all copies or substantial portions of the Software.
+%%%
+%%% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+%%% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+%%% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+%%% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+%%% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+%%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+%%% DEALINGS IN THE SOFTWARE.
+%%%
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2013-2017 Michael Truog
+%%% @version 1.7.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_reltool_util).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -export([application_env/1,
          application_start/1,
@@ -880,12 +866,13 @@ module_version(Module)
 script_start(FilePath)
     when is_list(FilePath) ->
     true = lists:suffix(".script", FilePath),
-    {ok,
-     [{script,
-       {_Name, _Vsn},
-       _Instructions} = Script]} = file:consult(FilePath),
-    Dir = filename:dirname(FilePath),
-    script_start_data(Script, Dir).
+    case file:consult(FilePath) of
+        {ok, [{script, {_Name, _Vsn}, _Instructions} = Script]} ->
+            Dir = filename:dirname(FilePath),
+            script_start_data(Script, Dir);
+        {error, _} = Error ->
+            Error
+    end.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -961,12 +948,13 @@ script_remove(FilePath, Timeout, Ignore)
          ((is_integer(Timeout) andalso (Timeout > 0)) orelse
           (Timeout =:= infinity)), is_list(Ignore) ->
     true = lists:suffix(".script", FilePath),
-    {ok,
-     [{script,
-       {_Name, _Vsn},
-       _Instructions} = Script]} = file:consult(FilePath),
-    Dir = filename:dirname(FilePath),
-    script_remove_data(Script, Dir, Timeout, Ignore).
+    case file:consult(FilePath) of
+        {ok, [{script, {_Name, _Vsn}, _Instructions} = Script]} ->
+            Dir = filename:dirname(FilePath),
+            script_remove_data(Script, Dir, Timeout, Ignore);
+        {error, _} = Error ->
+            Error
+    end.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -989,9 +977,13 @@ script_remove(FilePath, Timeout, Ignore)
 boot_start(FilePath)
     when is_list(FilePath) ->
     true = lists:suffix(".boot", FilePath),
-    {ok, ScriptData} = file:read_file(FilePath),
-    Dir = filename:dirname(FilePath),
-    script_start_data(erlang:binary_to_term(ScriptData), Dir).
+    case file:read_file(FilePath) of
+        {ok, ScriptData} ->
+            Dir = filename:dirname(FilePath),
+            script_start_data(erlang:binary_to_term(ScriptData), Dir);
+        {error, _} = Error ->
+            Error
+    end.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -1061,10 +1053,14 @@ boot_remove(FilePath, Timeout, Ignore)
          ((is_integer(Timeout) andalso (Timeout > 0)) orelse
           (Timeout =:= infinity)), is_list(Ignore) ->
     true = lists:suffix(".boot", FilePath),
-    {ok, ScriptData} = file:read_file(FilePath),
-    Dir = filename:dirname(FilePath),
-    script_remove_data(erlang:binary_to_term(ScriptData),
-                       Dir, Timeout, Ignore).
+    case file:read_file(FilePath) of
+        {ok, ScriptData} ->
+            Dir = filename:dirname(FilePath),
+            script_remove_data(erlang:binary_to_term(ScriptData),
+                               Dir, Timeout, Ignore);
+        {error, _} = Error ->
+            Error
+    end.
 
 %%%------------------------------------------------------------------------
 %%% Private functions
@@ -1466,8 +1462,10 @@ script_start_instructions([{apply,
                           applications_loaded, Applications, Root, Apps) ->
     case ensure_application_started(A) of
         ok ->
+            % order loaded applications based on the start order
             script_start_instructions(L, applications_loaded,
-                                      Applications, Root, Apps);
+                                      [A | lists:delete(A, Applications)],
+                                      Root, Apps);
         {error, _} = Error ->
             Error
     end;
@@ -1545,9 +1543,12 @@ script_remove_instructions([{apply, {application, load, [AppDescr]}} | L],
                            init_kernel_started, Applications) ->
     {application, A, [_ | _]} = AppDescr,
     script_remove_instructions(L, init_kernel_started, [A | Applications]);
-script_remove_instructions([{apply, {application, start_boot, _}} | L],
+script_remove_instructions([{apply,
+                             {application, start_boot, [A | _]}} | L],
                            applications_loaded, Applications) ->
-    script_remove_instructions(L, applications_loaded, Applications);
+    % order loaded applications based on the start order
+    script_remove_instructions(L, applications_loaded,
+                               [A | lists:delete(A, Applications)]);
 script_remove_instructions([{apply, {c, erlangrc, _}} | L],
                            applications_loaded, Applications) ->
     script_remove_instructions(L, applications_loaded, Applications).
@@ -1615,10 +1616,17 @@ is_module_loaded_check(Module) when is_atom(Module) ->
             false
     end.
 
+modules_unload([]) ->
+    ok;
+modules_unload([Module | Modules]) ->
+    _ = module_unload(Module),
+    modules_unload(Modules).
+
 modules_purged(Modules, infinity) ->
-    modules_purged(Modules, [], 5000);
+    modules_purged(Modules, 5000);
 modules_purged(Modules, Timeout)
     when is_list(Modules), is_integer(Timeout), Timeout >= 0 ->
+    ok = modules_unload(Modules),
     modules_purged(Modules, [], Timeout).
 
 modules_purged([], [], _) ->
