@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2018 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,13 +29,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2018 Michael Truog
+%%% @version 1.7.4 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(supool).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(gen_server).
 
@@ -172,13 +172,12 @@ handle_call(get, _, #state{supervisor = Supervisor,
     end;
 
 handle_call(Request, _, State) ->
-    ?LOG_WARN("Unknown call \"~p\"", [Request]),
-    {stop, lists:flatten(io_lib:format("Unknown call \"~p\"", [Request])),
+    {stop, lists:flatten(io_lib:format("Unknown call \"~w\"", [Request])),
      error, State}.
 
 handle_cast(Request, State) ->
-    ?LOG_WARN("Unknown cast \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, lists:flatten(io_lib:format("Unknown cast \"~w\"", [Request])),
+     State}.
 
 handle_info({start, ChildSpecs}, #state{supervisor = Supervisor} = State) ->
     case supool_sup:start_children(Supervisor, ChildSpecs) of
@@ -205,8 +204,8 @@ handle_info(restart, #state{supervisor = Supervisor} = State) ->
     end;
 
 handle_info(Request, State) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, lists:flatten(io_lib:format("Unknown info \"~w\"", [Request])),
+     State}.
 
 terminate(_, _) ->
     ok.
