@@ -17,14 +17,15 @@ REBAR_DIRS="\
     lib/cloudi_service_http_rest \
     lib/cloudi_service_map_reduce \
     lib/cloudi_service_monitoring \
+    lib/cloudi_service_null \
     lib/cloudi_service_oauth1 \
     lib/cloudi_service_queue \
     lib/cloudi_service_quorum \
+    lib/cloudi_service_request_rate \
     lib/cloudi_service_router \
     lib/cloudi_service_tcp \
     lib/cloudi_service_udp \
     lib/cloudi_service_validate \
-    lib/cloudi_service_zeromq \
     external/cloudi_x_bear \
     external/cloudi_x_certifi \
     external/cloudi_x_cowboy \
@@ -49,6 +50,7 @@ REBAR_DIRS="\
     external/cloudi_x_ranch \
     external/cloudi_x_setup \
     external/cloudi_x_ssl_verify_fun \
+    external/cloudi_x_unicode_util_compat \
     external/proper \
     lib/cgroups \
     lib/cpg \
